/*
**> pde2d3.prg
**
*/

new;
library PDE2D,option,pgraph;
PDE2Dset;

K = 100;
sigma = 0.20;
tau = 0.25;
r = 0.08;
b = 0.08;

proc aProc(t,x,y);
  retp( 0.5*(sigma^2) .* (x^2) .* ones(1,cols(y)) );
endp;

proc bProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc cProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc dProc(t,x,y);
  retp( b .* x .* ones(1,cols(y)) );
endp;

proc eProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( r .* ones(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff .* ones(1,cols(y)) );
endp;

proc xminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DxmaxBound(t,x,y);
  retp( ones(rows(x),cols(y)) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
               &tminBound,&xminBound,0,0,0,
                          0,&DxmaxBound,0,0);

xmin = 50; xmax = 150; Nx = 201;
ymin = 0.10; ymax = 0.30; Ny = 5;
tmin = 0; tmax = tau; Nt = floor(1825*tau);

_Hopscotch_PrintIters = 10;
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"pde2d3");

dt = vread(_Hopscotch_MeshRatios,"k");
dS = vread(_Hopscotch_MeshRatios,"hx");

indxT = seqa(10,15,30);
indxS0 = seqa(1,10,21);

graphset;

  begwind; 
  window(2,2,0); 
   
  _pdate = ""; _pnum = 2; _pnumht = 0.25; _ptitlht = 0.25; _paxht = 0.25; 
  fonts("simplex simgrma");

  setwind(1); 
   
    S0 = ExtractSolution("pde2d3","x");
    U = ExtractSolution("pde2d3","t"|tau);
    delta = Derive(U,dS,0);

    title("Delta");
    xlabel("S0");
    ytics(0,1,0.25,0);
    xy(S0[indxS0],submat(delta~EuropeanBS_Delta(S0,K,sigma,tau,b,r),indxS0,0));

  setwind(2); 

    gamma_ = Derive(Derive(U,dS,-1),dS,1);

    title("Gamma");
    ytics(0,0.05,0.01,0);
    xy(S0[indxS0],submat(gamma_~EuropeanBS_Gamma(S0,K,sigma,tau,b,r),indxS0,0));

  setwind(3); 

    t = ExtractSolution("pde2d3","t");
    y = ExtractSolution("pde2d3","y");
    U = ExtractSolution("pde2d3","y"|y[3]);
    theta = Derive(U',dt,0);

    title("Theta (Hopscotch)");
    xlabel("\202t\201");
    ylabel("S0");
    xtics(0,0.20,0.10,0);
    ytics(80,120,10,0);
    ztics(0,50,10,0);
    surface(submat(t',0,indxT),submat(S0,indxS0,0),
            submat(theta',indxS0,indxT));
 
  setwind(4); 

    title("Theta");
    surface(submat(t',0,indxT),submat(S0,indxS0,0),
            submat(EuropeanBS_Theta(S0,K,sigma,t',b,r),indxS0,indxT));

    graphprt("-c=1 -cf=pde2d3.eps");  

  endwind; 
