/*
** Kurpiel, A. and T. Roncalli [1998], Hopscotch methods for two-state
** financial models, FERC working paper, City University Business School
*/


new;
library option,PDE2D,pgraph;
PDE2Dset;

K = 100;
S0 = seqa(80,5,9);
b = -0.04;
r =  0.08;
V0 = 0.20^2;
tau = 0.25;

rho = -0.5;
Kappa = 0.9;
theta = 0.20^2;
lambda = 0.0;
sigmaV = 0.10;
kappaStar = kappa + lambda;
thetaStar = kappa*theta/(kappa+lambda);

load MeshRatios = ratios;

S = ExtractSolution("pde2d4","x");
V = ExtractSolution("pde2d4","y");
tau = ExtractSolution("pde2d4","t");
U = ExtractSolution("pde2d4","t"|tau);

dS = vread(MeshRatios,"hx");
dV = vread(MeshRatios,"hy");

Delta1 = Derive(U,dS,0);
Vega = Derive(U',dV,0)';
Delta2 = EuropeanBS_Delta(S,K,sqrt(V),tau,b,r);


graphset;
  _pdate = ""; _pnum = 2; 
  _pltype = 6;
  fonts("simplex simgrma");
  title("\202D\201]am,Hopscotch[");
  xy(S,delta1);

graphset;
  _pdate = ""; _pnum = 2; 
  _pltype = 6;
  fonts("simplex simgrma");
  title("\202D\201]am,Hopscotch[ - \202D\201]eu,BS[");
  ylabel("S]0[");
  xlabel("V]0[");
  surface(trimr(V',5,5)',S,trimr((Delta1-Delta2)',5,5)');

graphset;
  _pdate = ""; _pnum = 2; 
  title("VEGA]am,Hopscotch[");
  ztics(0,100,25,0);
  xlabel("S]0[");
  ylabel("V]0[");
  graphprt("-c=1 -cf=pde2d5.eps");
  surface(S',V',Vega');