/*
** Kurpiel, A. and T. Roncalli [1998], Hopscotch methods for two-state
** financial models, FERC working paper
**
** Term Structure Modelling
** 
**
*/

/*
new;
library pde2D,pgraph;
PDE2Dset;

#include vasicek.src;

aR = 1.3; bR = 0.15; sigmaR = 0.25; lambdaR = -0.3;
aL = 0.8; bL = 0.10; sigmaL = 0.15; lambdaL = 0.0;
alpha = 0.0; rho = 0.5;

tau = 1;

proc aProc(t,x,y);
  retp( 0.5*(sigmaR^2) .* ones(rows(x),cols(y)) );
endp;

proc bProc(t,x,y);
  retp( 0.5*rho*sigmaR*sigmaL .* ones(rows(x),cols(y)) );
endp;

proc cProc(t,x,y);
  retp( 0.5*(sigmaL^2) .* ones(rows(x),cols(y)) );
endp;

proc dProc(t,x,y);
  retp( (aR*(bR-x) + alpha*(bL-y) - lambdaR*sigmaR) .* ones(rows(x),cols(y)) );
endp;

proc eProc(t,x,y);
  retp( (aL*(bL-y) - lambdaL*sigmaL) .* ones(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( x .* ones(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  retp( ones(rows(x),cols(y)) );
endp;

proc DxminBound(t,x,y);
  local P,R;
  {P,R} = Vasicek(x,aR,bR,sigmaR,lambdaR,t);
  retp( -P.*(1-exp(-aR*t))/aR.*ones(rows(x),cols(y)));
endp;

proc DxmaxBound(t,x,y);
  local P,R;
  {P,R} = Vasicek(x,aR,bR,sigmaR,lambdaR,t);
  retp( -P.*(1-exp(-aR*t))/aR.*ones(rows(x),cols(y)) );
endp;

proc DyminBound(t,x,y);
  retp( 0*ones(rows(x),cols(y)) );
endp;

proc DymaxBound(t,x,y);
  retp( 0*ones(rows(x),cols(y)) );
endp;


xmin = 0.000; xmax = 0.300; Nx = 31;
ymin = 0.000; ymax = 0.300; Ny = 31;
tmin = 0; tmax = tau; Nt = floor(1825*tau);

__output = 1;

_Hopscotch_SaveLastIter = 0;
_Hopscotch_PrintIters = 10;


call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                 &tminBound,0,0,0,0,
                 &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ts1");

x = ExtractSolution("ts1","x");
y = ExtractSolution("ts1","y");
tau = ExtractSolution("ts1","t");

indx = (Nx+1)/2;
r0 = x[indx];
l0 = y[indx];

U = ExtractSolution("ts1","y"|l0);
U0 = U[indx,.]';

alpha = 0.25;
call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                 &tminBound,0,0,0,0,
                 &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ts2");

U = ExtractSolution("ts2","y"|l0);
U1 = U[indx,.]';

alpha = 0.25; bL = 0.20;
call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                 &tminBound,0,0,0,0,
                 &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ts3");

U = ExtractSolution("ts3","y"|l0);
U2 = U[indx,.]';


alpha = 0.25; bL = 0.20; aL = 1.5;
call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                 &tminBound,0,0,0,0,
                 &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ts4");

U = ExtractSolution("ts4","y"|l0);
U3 = U[indx,.]';

U = U0~U1~U2~U3;
{P,R} = Vasicek(r0,aR,bR,sigmaR,lambdaR,tau);

save tau, R, P, U;

*/
load tau, R, P, U;


graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(3.5,2.5,1.4,4,1);

  setwind(1);
    _pnum = 2; _pdate = "";
    _pltype = 6|6|6|6|6;
    _pstype = 1|1|9|10|8;
    _plctrl = 0|100|100|100|100;
    fonts("simplex simgrma");
    xlabel("\202t\201");  
    ylabel("R\202\34t\118\201");
    ytics(0.150,0.175,0.005,0);
    str = "Vasicek solution\000" $+
           ftos(0,"\202a\201 = %lf",4,2) $+ "\000" $+
           ftos(0.25,"\202a\201 = %lf",4,2) $+ ftos(0.10," b]l[ = %lf",4,2) $+
           ftos(0.08," \201a\201]l[ = %lf",4,2) $+ "\000" $+
           ftos(0.25,"\202a\201 = %lf",4,2) $+ ftos(0.20," b]l[ = %lf",4,2) $+
           ftos(0.08," \201a\201]l[ = %lf",4,2) $+ "\000" $+
           ftos(0.25,"\202a\201 = %lf",4,2) $+ ftos(0.20," b]l[ = %lf",4,2) $+
           ftos(1.50," \201a\201]l[ = %lf",4,2);
    _plegstr = str;
    _plegctl = {2 5 4 1.5};

    xy(tau,R~-ln(U)./tau);

  setwind(2);
    graphset;
    _pnum = 2; _pnumht = 0.25; _ptitlht = 0.30;
    _pltype = 6|6|6|6|6;
    _pstype = 1|1|9|10|8;
    _plctrl = 0|100|100|100|100;
    fonts("simplex simgrma");
    title("P]c[\202\34t\118\201");
    ytics(0.80,1,0.05,0);
    xy(tau,P~U);

  graphprt("-c=1 -cf=wp7.eps");
  endwind;




